/*
################################################################################
#  THIS FILE IS 100% GENERATED BY ZPROJECT; DO NOT EDIT EXCEPT EXPERIMENTALLY  #
#  Read the zproject/README.md for information about making permanent changes. #
################################################################################
*/
package org.zeromq.czmq;

import org.junit.Assert;
import org.junit.Test;
import org.scijava.nativelib.NativeLoader;

public class ZhashTest {

    @Test
    public void test () {
        Zhash.test (false);
    }

}
